"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __exportStar = (this && this.__exportStar) || function(m, exports) {
    for (var p in m) if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AuthBaseToken = exports.schemaBaseIdToken = exports.TokenKind = void 0;
const joi_1 = __importDefault(require("joi"));
const jsonwebtoken_1 = __importDefault(require("jsonwebtoken"));
const cm_libs_context_aware_error_1 = require("cm-libs-context-aware-error");
const auth_type_1 = require("./auth-type");
__exportStar(require("jsonwebtoken"), exports);
var TokenKind;
(function (TokenKind) {
    TokenKind["TOKEN"] = "TOKEN";
    TokenKind["REFRESH_TOKEN"] = "REFRESH_TOKEN";
})(TokenKind = exports.TokenKind || (exports.TokenKind = {}));
const _id = joi_1.default.number();
const _appDomain = joi_1.default.string()
    .trim()
    .valid(...Object.values(auth_type_1.AppDomain));
const _userName = joi_1.default.string().max(250);
const _localeCode = joi_1.default.string().max(10);
exports.schemaBaseIdToken = {
    appDomain: _appDomain.required(),
    userId: _id.required(),
    userName: _userName.required(),
    productEntityId: _id,
    roleId: _id,
    localeCode: _localeCode.required()
};
class AuthBaseToken {
    constructor(secret, expiresIn, schema, kind, algorithm) {
        this.secret = secret || (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.PARAMETER_MISSING, 'Auth token secret is missing');
        this.schema = schema;
        this.kind = kind;
        this.signOptions = {
            expiresIn
        };
        if (undefined !== algorithm) {
            this.signOptions.algorithm = algorithm;
        }
    }
    sign(payload, expiresIn, notBefore) {
        const { error, value: claim } = this.schema.validate(payload, { stripUnknown: true });
        if (error) {
            (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.INVALID_FORMAT, error.message, {
                function: 'sign',
                details: error.details,
                text: { input: error.details?.[0]?.context?.peers || error.details?.[0]?.context?.key }
            });
        }
        if (this.kind !== claim['kind']) {
            (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.MISMATCHED, 'Token kind is mismatched', { claim });
        }
        const signOptions = this.signOptions;
        if (undefined !== expiresIn) {
            signOptions.expiresIn = expiresIn;
        }
        if (undefined !== notBefore) {
            signOptions.notBefore = notBefore;
        }
        return jsonwebtoken_1.default.sign(claim, this.secret, signOptions);
    }
    verify(token) {
        let baseToken;
        try {
            baseToken = jsonwebtoken_1.default.verify(token, this.secret);
        }
        catch (error) {
            switch (error?.name) {
                case 'TokenExpiredError':
                    (0, cm_libs_context_aware_error_1.throwFromError)(error, cm_libs_context_aware_error_1.errors.EXPIRED, { token }, false);
                case 'NotBeforeError':
                    (0, cm_libs_context_aware_error_1.throwFromError)(error, cm_libs_context_aware_error_1.errors.NOT_READY, { token }, false);
            }
            (0, cm_libs_context_aware_error_1.throwFromError)(error, cm_libs_context_aware_error_1.errors.UNAUTHORISED, { token }, false);
        }
        const { error, value: claim } = this.schema.validate(baseToken, { stripUnknown: true });
        /* istanbul ignore next */
        if (error) {
            (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.INVALID_FORMAT, error.message, {
                function: 'verify',
                details: error.details,
                text: { input: error.details?.[0]?.context?.peers || error.details?.[0]?.context?.key }
            });
        }
        if (this.kind !== claim['kind']) {
            (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.MISMATCHED, 'Token kind is mismatched', { claim });
        }
        return claim;
    }
}
exports.AuthBaseToken = AuthBaseToken;
//# sourceMappingURL=auth-base-token.js.map