"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.hasAbility = exports.isAuthenticated = exports.auth = void 0;
const ability_1 = require("@casl/ability");
const cm_libs_context_aware_error_1 = require("cm-libs-context-aware-error");
const http_1 = require("../http");
const auth_type_1 = require("./auth-type");
const auth_bearer_1 = require("./auth-bearer");
const auth_basic_1 = require("./auth-basic");
async function auth(req, authDirective) {
    const authToken = (0, http_1.getAuthToken)(req);
    if (authToken?.info?.token) {
        if (undefined === authToken.grant) {
            let _authToken;
            switch (authDirective.type) {
                case auth_type_1.AuthTokenType.Bearer: {
                    _authToken = await auth_bearer_1.AuthBearer.instance.verifyIdToken(authToken);
                    // Generate the ability into auth token and set into req object
                    (0, http_1.setAuthToken)(req, _authToken);
                    return _authToken;
                }
                case auth_type_1.AuthTokenType.Basic: {
                    const loginInfo = authToken.info.authMetadata;
                    const _authToken = await auth_basic_1.AuthBasic.instance.verifyIdToken(authToken, loginInfo?.username, loginInfo?.password, authToken.info.headers?.appDomain);
                    // Generate the ability into auth token and set into req object
                    (0, http_1.setAuthToken)(req, _authToken);
                    return _authToken;
                }
            }
        }
        if (true === authToken.grant) {
            return authToken;
        }
    }
    (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.UNAUTHORISED, 'Unauthorized access', { hasToken: undefined !== authToken?.info?.token, spec: authDirective });
}
exports.auth = auth;
function isAuthenticated(authToken) {
    return true === authToken?.grant;
}
exports.isAuthenticated = isAuthenticated;
function hasAbility(authToken, rules) {
    if (!isAuthenticated(authToken) || !rules) {
        (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.UNAUTHORISED, 'Unauthorized access', { authToken, rules });
    }
    const ability = authToken.info.auth.ability;
    if (!Array.isArray(rules)) {
        return ability_1.ForbiddenError.from(ability).throwUnlessCan(rules.action, rules.subject);
    }
    for (const rule of rules) {
        ability_1.ForbiddenError.from(ability).throwUnlessCan(rule.action, rule.subject);
    }
}
exports.hasAbility = hasAbility;
//# sourceMappingURL=auth.js.map