import { PrismaClient } from '@prisma/client';
import { AccessInfo } from '../auth';
declare type MiddlewareParams = {
    model?: string;
    action: string;
    args: any;
    dataPath: string[];
    runInTransaction: boolean;
};
declare type ModelBrowseArgs = {
    pagination?: ModelBrowsePagination;
    select?: any;
    where?: any;
    orderBy?: any;
    distinct?: any;
    take?: number;
    skip?: number;
    options?: ModelBrowseOptions;
};
export declare type ModelBatchPayload = {
    count: number;
};
export declare type ModelErrorHandler = (error: TypeError, params: MiddlewareParams) => never;
export interface ModelBrowsePagination {
    pageSize?: number;
    page?: number;
}
export declare type ModelBrowseOptions = {
    speedMode?: boolean;
};
export declare class ModelMiddleware<T, Client, CustomCreateArgs, CustomCreateManyArgs, CustomUpdateArgs, CustomUpdateManyArgs, CustomBrowseArgs extends ModelBrowseArgs> {
    readonly model: Client;
    protected readonly accessInfo: AccessInfo;
    constructor(client: PrismaClient, model: string, accessInfo: AccessInfo, errorHandler?: ModelErrorHandler);
    private packCreateArgs;
    private packUpdateArgs;
    create(params: CustomCreateArgs): Promise<T>;
    createMany(params: CustomCreateManyArgs): Promise<ModelBatchPayload>;
    update(params: CustomUpdateArgs): Promise<T>;
    updateMany(params: CustomUpdateManyArgs): Promise<ModelBatchPayload>;
    browse(params: CustomBrowseArgs): Promise<{
        items: T[];
        properties: ModelBrowsePagination | {
            count: number;
            pageSize?: number;
            page?: number;
        };
    }>;
}
export {};
