import { AccessInfo } from '../auth';
export interface IPagination {
    pageSize?: number;
    page?: number;
}
export declare type BaseBrowse = {
    pagination?: IPagination;
    sort?: any;
    speedMode?: boolean;
};
declare type Dict = {
    [k: string]: any;
};
declare type DictWithId = {
    id?: number | bigint;
    [k: string]: any;
};
declare type SelectWithId = {
    id?: boolean;
    [k: string]: any;
};
interface BasePrismaClient {
    findMany: (arg: {
        select?: SelectWithId | null;
        include?: Dict | null;
        where?: Dict;
        orderBy?: unknown;
        cursor?: Dict;
        take?: number;
        skip?: number;
        distinct?: unknown;
    }) => any;
    create: (arg: {
        select?: SelectWithId | null;
        include?: Dict | null;
        data: any;
    }) => any;
    update: (arg: {
        select?: SelectWithId | null;
        include?: Dict | null;
        data: any;
        where: DictWithId;
    }) => any;
    delete: (arg: {
        select?: SelectWithId | null;
        include?: Dict | null;
        where: DictWithId;
    }) => any;
    [k: string]: any;
}
export declare type FormatErrorHandler = (error: TypeError, modelName: string, fnName: string, args: any[]) => never;
export declare type BaseOptions = {
    accessInfo: AccessInfo;
};
export declare type AddOptions = BaseOptions;
export declare type ReadOptions = BaseOptions;
export declare type EditOptions = BaseOptions;
export declare type DeleteOptions = BaseOptions;
export declare type ActivateOptions = BaseOptions;
export declare type DeactivateOptions = BaseOptions;
export declare type BrowseOptions = BaseOptions & {
    scanAll?: boolean;
    sort?: object;
    speedMode?: boolean;
};
export declare abstract class BaseModel<PrismaClient extends BasePrismaClient, CustomCreateInput, WhereUniqueInput, WhereInput, CustomUpdateInput, R = unknown> {
    protected readonly prisma: PrismaClient;
    protected readonly modelName: string;
    protected readonly formatErrorHandler: FormatErrorHandler;
    constructor(prisma: PrismaClient, formatErrorHandler: FormatErrorHandler, modelName: string);
    add(input: CustomCreateInput, options?: AddOptions): Promise<R>;
    read(where: WhereUniqueInput, options?: ReadOptions): Promise<R>;
    browse(pagination?: IPagination, where?: WhereInput, options?: BrowseOptions): Promise<{
        items: R[];
        properties: {
            count?: number;
            pageSize?: number;
            page?: number;
        };
    }>;
    edit(input: CustomUpdateInput, where: WhereUniqueInput, options?: EditOptions): Promise<R>;
    delete(where: WhereUniqueInput, options?: DeleteOptions): Promise<R>;
    activate(where: WhereUniqueInput, options?: ActivateOptions): Promise<R>;
    deactivate(where: WhereUniqueInput, options?: DeactivateOptions): Promise<R>;
}
export {};
